/* spi-MMA7455.c  
    Accelerometer interface using spiasm library
 Based on C-Learning SPI tutorial by Andy Lindsay
  
 Uses pasm in xspiasm.spin, modified from SPI_Asm.spin propeller tool library object.
  T Montemarano  (MIT License)
  
  The SPI device called in main() is a MMA7455 Accelerometer.    
*/

#include "simpletools.h"                      // Library includes
#include "spiasm.h"
  
/* ******************************************************
   Uses MMA7455 Accelerometer as SPI device

    MMA7455 pin              Prop pin 

         CS                 <-    Prop P6  low starts transfer, high terminates
         DATA in/out   <->  Prop P7. Prop out (MFIRST), Prop in (MPRE)
         Clock             <-    Prop P8  rise then fall  State = 0

*/

#define cstate 0               // 0 = start clock low, 1 = start clock high
#define clockd_ns 1000   // Actual delay in nanoseconds (1000ns = 1 usec), not less than 300ns
                                      // ClockDelay, actual clock delay = 300ns + (cdelay-1) * 50ns
                                      // e.g. cdelay of 15 gives actual clock delay = 1 usec
                                      // cdelay = ((clockd_ns) - 250ns)) / 50ns
                                      // e.g. for 2 usec (i.e. 2000 ns): cdelay = 35
//  ** Propeller pins 
#define CS 6
#define IOPIN 7
#define CLKPIN 8

spia_t *spia; 
signed char xval, yval, zval;
  
int main()                                    // Main function
{ 
// ** MMA7455 commands
  int wmctl = 0x16 | 0x40;   // Write MCTL register. address 0x16 | (bit 7 WRITE) = 0b1010110
  int mctlval = 0x65;   // Value for MCTL register = 0b01100101;
        // D7=0, DRPD=1, SPI3W=1, STON=0, GLVL[1]=0, GLVL[0]=1, MOD[1]=0, MOD[0]=1
        // Don't care, no interupt, SPI 3 wire, no self test, GLVL 2g, MOD measurement on

  int i2cad =0x0D | 0x40;   // address of I2CAD register (write set)
  int i2cdis = 0x80;              // high in 8th bit disables i2c

  int rzout = 0x08;    // Command to Read ZOUT register  0x08 (bit 7 is READ)
  int rxout = 0x06;
  int ryout = 0x07;

  high(CS);                                        // disable MMA7455 data transfer
  spia = spi_start(clockd_ns, cstate);    // Launch, get handles, set ClockDelay & ClockState
  low(CS);                                         // enable MMA7455 data transfer

          // first disable i2c
            // void spi_out(spia_t *device, int DQ, int CLK, int MD, int BIT, int VLU);
  spi_out(spia, IOPIN, CLKPIN, MFIRST, 7, i2cad);            // request i2cad write
  spi_out(spia, IOPIN, CLKPIN, MFIRST, 1, 0);                 //  send don't care bit
  spi_out(spia, IOPIN, CLKPIN, MFIRST, 8, i2cdis);           // disable I2C
  pause(1);               // needed before disable data transfer is sent
  high(CS);                // release MMA7455

  low(CS);
          // set configuration register MCTL
  spi_out(spia, IOPIN, CLKPIN, MFIRST, 7, wmctl);   // request Confguration write
  spi_out(spia, IOPIN, CLKPIN, MFIRST, 1, 0);   //  send don't care bit
  spi_out(spia, IOPIN, CLKPIN, MFIRST, 8, mctlval);    // Send value for MCTL
  pause(1);              
  high(CS);                // release MMA7455
  pause(1);                // may not need 

  while(1)  
  {
    low(CS);
    spi_out(spia, IOPIN, CLKPIN, MFIRST, 7, rxout);    // request to Read the X acceleration
    spi_out(spia, IOPIN, CLKPIN, MFIRST, 1, 0);         //  send don't care bit
    xval = spi_in(spia, IOPIN, CLKPIN, MPRE, 8);        // read X axis acceleration
    pause(1);       
    high(CS);                // release MMA7455

    low(CS);
    spi_out(spia, IOPIN, CLKPIN, MFIRST, 7, ryout);     // request to Read the y acceleration
    spi_out(spia, IOPIN, CLKPIN, MFIRST, 1, 0);          //  send don't care bit
    yval = spi_in(spia, IOPIN, CLKPIN, MPRE, 8);        // read y axis acceleration  
    pause(1);
    high(CS);                // release MMA7455

    low(CS);
    spi_out(spia, IOPIN, CLKPIN, MFIRST, 7, rzout);    // request to Read the Z acceleration
    spi_out(spia, IOPIN, CLKPIN, MFIRST, 1, 0);         //  send don't care bit
    zval = spi_in(spia, IOPIN, CLKPIN, MPRE, 8);       // read Z axis acceleration
    pause(1);               
    high(CS);                // release MMA7455

    printi(" %c x = %d%c\n", HOME, xval, CLREOL);
    printi("  y = %d%c\n",  yval, CLREOL);
    printi("  z = %d%c\n",  zval, CLREOL);

    pause(500);
  }
}   // end main
